﻿using System.Collections.Generic;
using System.Data;
using System.Linq;
using AdventureWorksModel;

namespace ServerSideEFMVC3Razor
{
  public class DataAccess
  {
   
    public static PagedList<Customer> GetPagedCustomers(int skip, int take)
    {
      using (var context = new AdventureWorksLTEntities())
      {

        //refactor consideration...make this a pre-compiled query
        var query = context.Customers.Include("SalesOrderHeaders")
          .Where(c => c.SalesOrderHeaders.Any())
          .OrderBy(c => c.CompanyName + c.LastName + c.FirstName);

        //getting this count every time in case others are adding or deleting customers
        //decide for yourself in your app
        var customerCount = query.Count();

        var customers = query.Skip(skip).Take(take).ToList();
      
      return new PagedList<Customer>
        {
         Entities = customers,
          HasNext = (skip + 10 < customerCount),
          HasPrevious = (skip > 0)
        };
        }
      }
    
  public static Customer GetCustomerById(int id)
    {
      using (var context = new AdventureWorksLTEntities())
      {
        //refactor consideration...make this a pre-compiled query
        var query = from c in context.Customers
                    where c.CustomerID == id
                    select c;
        return query.SingleOrDefault(); //non-demo app should consider exception handling
      }
    }
    public static void UpdateCustomer(Customer customer)
    {
      //this update is simplified because I placed some non-nullable properties
      //that I don't want to work with into a separate entity using the "table splitting"
      //mapping feature of EDM. If you were doing an insert, you'll have to deal with those
      //properties from the related entity. Way beyond the scope of the article that this
      //demo is for, however.
      using (var context = new AdventureWorksLTEntities())
      {
        context.Customers.Attach(customer);
        context.ObjectStateManager.ChangeObjectState(customer, EntityState.Modified);
        context.SaveChanges(); //non-demo app should consider exception handling
      }
    }
   }
  
  public class PagedList<T>
  {
    public bool HasNext { get; set; }
    public bool HasPrevious { get; set; }
    public List<T> Entities { get; set; }
  }

}